import asyncio
from selve.communication import *
from selve.protocol import *
import logging
_LOGGER = logging.getLogger(__name__)

class Device():

    def __init__(self, gateway, ID, deviceType:DeviceType = DeviceType.UNKNOWN):
        self.ID = ID
        self.gateway = gateway
        self.mask = singlemask(ID)
        self.device_type = deviceType
        self.name = "Not defined"
        self.deviceClass = DeviceClass.UNKOWN
        self.communicationType = CommunicationType.UNKNOWN
    
    async def discover_properties(self):
        pass

    def __str__(self):
        return "Device " + self.device_type.name + " of type: " + self.communicationType.name + " on channel " + str(self.ID) + " with name " + self.name


import asyncio
from enum import Enum
from os import wait
from selve import commands
from selve import utils
from selve.commandClasses.command import CommeoCommandDevice

from selve.protocol import CommandType, CommunicationType, DayMode, DeviceClass, DeviceCommandTypes, DeviceState, MethodCall, MovementState, ScanState, ServiceState
from selve.protocol import ParameterType
from selve.protocol import DeviceType
from selve.commands import Commands, CommeoCommandCommand, CommeoDeviceCommand, CommeoEventCommand, CommeoGroupCommand, CommeoParamCommand, CommeoSenSimCommand, CommeoSenderCommand, CommeoSensorCommand, CommeoServiceCommand
from selve.utils import intToBoolarray, singlemask, valueToPercentage
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
from selve.communication import Command, CommandSingle
from selve.device import Device
import logging
_LOGGER = logging.getLogger(__name__)



class CommeoDeviceScanStart(Command):
    def __init__(self):
        super().__init__(CommeoDeviceCommand.SCANSTART)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
class CommeoDeviceScanStop(Command):
    def __init__(self):
        super().__init__(CommeoDeviceCommand.SCANSTOP)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
class CommeoDeviceScanResult(Command):
    def __init__(self):
        super().__init__(CommeoDeviceCommand.SCANRESULT)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.scanState = ScanState(int(methodResponse.parameters[0][1]))
        self.noNewDevices = int(methodResponse.parameters[1][1])
        self.foundIds = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[2][1]))]

class CommeoDeviceSave(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoDeviceCommand.SAVE, deviceId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoDeviceGetIDs(Command):
    def __init__(self):
        super().__init__(CommeoDeviceCommand.GETIDS)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)

class CommeoDeviceGetInfo(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoDeviceCommand.GETINFO, deviceId)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = methodResponse.parameters[0][1]
        self.rfAddress = methodResponse.parameters[2][1]
        self.deviceType = DeviceType(int(methodResponse.parameters[3][1]))
        self.state = DeviceState(int(methodResponse.parameters[4][1]))

class CommeoDeviceGetValues(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoDeviceCommand.GETVALUES, deviceId)


    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = methodResponse.parameters[0][1]
        self.movementState = MovementState(int(methodResponse.parameters[2][1]))
        self.value = valueToPercentage(int(methodResponse.parameters[3][1]))
        self.targetValue = valueToPercentage(int(methodResponse.parameters[4][1]))

        bArr = intToBoolarray(int(methodResponse.parameters[5][1]))
        self.unreachable = bArr[0]
        self.overload = bArr[1]
        self.obstructed = bArr[2]
        self.alarm = bArr[3]
        self.lostSensor = bArr[4]
        self.automaticMode = bArr[5]
        self.gatewayNotLearned = bArr[6]
        self.windAlarm = bArr[7]
        self.rainAlarm = bArr[8]
        self.freezingAlarm = bArr[9]
        
        self.dayMode = DayMode(int(methodResponse.parameters[6][1]))

class CommeoDeviceSetFunction(Command):
    def __init__(self, deviceId, function):
        super().__init__(CommeoDeviceCommand.SETFUNCTION, [(ParameterType.INT, deviceId), (ParameterType.INT, function)])

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoDeviceSetLabel(Command):
    def __init__(self, deviceId, name):
        super().__init__(CommeoDeviceCommand.SETLABEL, [(ParameterType.INT, deviceId), (ParameterType.STRING, name)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoDeviceSetType(Command):
    def __init__(self, deviceId, type):
        super().__init__(CommeoDeviceCommand.SETTYPE, [(ParameterType.INT, deviceId), (ParameterType.INT, type.value)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoDeviceDelete(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoDeviceCommand.DELETE, deviceId)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
    
class CommeoDeviceWriteManual(Command):
    def __init__(self, deviceId, rfAddress, name, deviceType):
        super().__init__(CommeoDeviceCommand.WRITEMANUAL, [(ParameterType.STRING, name), (ParameterType.INT, deviceId), (ParameterType.INT, rfAddress), (ParameterType.INT, deviceType.value)])

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])


class ActorDevice(Device):
    def __init__(self, gateway, id):
        super().__init__(gateway, id)
        self.communicationType = CommunicationType.COMMEO
        self.deviceClass = DeviceClass.ACTOR

    async def discover_properties(self):
        try:
            command = CommeoDeviceGetInfo(self.ID)
            await command.execute(self.gateway)
            self.device_type = command.deviceType
            self.name = command.name
            self.rfAddress = command.rfAddress
            self.state = command.state
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))

    
    async def getDeviceValues(self):
        command = CommeoDeviceGetValues(self.ID)
        await command.execute(self.gateway)
        
        self.name = command.name
        self.movementState = command.movementState
        self.value = command.value
        self.targetValue = command.targetValue
        self.unreachable = command.unreachable
        self.overload = command.overload
        self.obstructed = command.obstructed
        self.alarm = command.alarm
        self.lostSensor = command.lostSensor
        self.automaticMode = command.automaticMode
        self.gatewayNotLearned = command.gatewayNotLearned
        self.windAlarm = command.windAlarm
        self.rainAlarm = command.rainAlarm
        self.freezingAlarm = command.freezingAlarm
        self.dayMode = command.dayMode
        return command

    async def setDeviceFunction(self, func):
        command = CommeoDeviceSetFunction(self.ID, func)
        await command.execute(self.gateway)

    async def setDeviceLabel(self):
        command = CommeoDeviceSetLabel(self.ID, self.name)
        await command.execute(self.gateway)

    async def setDeviceType(self):
        command = CommeoDeviceSetType(self.ID, self.device_type)
        await command.execute(self.gateway)

    async def deleteDevice(self):
        command = CommeoDeviceDelete(self.ID)
        await command.execute(self.gateway)
        self.gateway.deleteDevice(self.ID)

    async def setDeviceManual(self):
        command = CommeoDeviceWriteManual(self.ID, self.rfAddress, self.name, self.device_type)
        await command.execute(self.gateway)

    async def executeCommand(self, command, commandType = DeviceCommandTypes.MANUAL, parameter = 0):
        command = CommeoCommandDevice(self.ID, command, commandType, parameter)
        await command.execute(self.gateway)
        return command

    async def stop(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.STOP, type)

    async def moveDown(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.DRIVEDOWN, type)
    
    async def moveUp(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.DRIVEUP, type)
    
    async def moveIntermediatePosition1(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.DRIVEPOS1, type)

    async def moveIntermediatePosition2(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.DRIVEPOS2, type)
    
    async def driveToPos(self, position, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.DRIVEPOS, type, utils.percentageToValue(position))

    async def stepUp(self, degrees, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.STEPUP, type, degrees)

    async def stepDown(self, degrees, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        await self.executeCommand(CommandType.STEPDOWN, type, degrees)

    async def setAutomatic(self, autoOn, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        if autoOn:
            await self.executeCommand(CommandType.AUTOON, type)
        else:
            await self.executeCommand(CommandType.AUTOOFF, type)

    async def saveDevice(self):
        command = CommeoDeviceSave(self.ID)
        await command.execute(self.gateway)
        if command.executed:
            _LOGGER.info("Device saved")
        else:
            _LOGGER.info("Device saving failed")

    async def deleteDevice(self):
        command = CommeoDeviceDelete(self.ID)
        await command.execute(self.gateway)
        if command.executed:
            _LOGGER.info("Device deleted")
        else:
            _LOGGER.info("Device deletion failed")



import asyncio
from selve.device import Device

from selve.protocol import CommandTypeIveo, CommunicationType, DeviceClass, MethodCall, RepeaterState
from selve.protocol import ParameterType
from selve.protocol import DeviceType
from selve.commands import IveoCommand
from selve.communication import Command, CommandMask, CommandSingle
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
import logging


_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)

    
class IveoCommandSetRepeater(Command):
    def __init__(self, repeaterState):
        super().__init__(IveoCommand.SETREPEATER, [(ParameterType.INT, repeaterState)])

class IveoCommandGetRepeater(Command):
    def __init__(self):
        super().__init__(IveoCommand.GETREPEATER)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.repeaterState = RepeaterState(int(methodResponse.parameters[0][1]))

class IveoCommandFactory(CommandSingle):

    def __init__(self, iveoID):
        super().__init__(IveoCommand.FACTORY, iveoID)

class IveoCommandTeach(CommandSingle):
    def __init__(self, iveoID):
        super().__init__(IveoCommand.TEACH, iveoID)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])   

class IveoCommandLearn(CommandSingle):
    def __init__(self, iveoID):
        super().__init__(IveoCommand.LEARN, iveoID)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1]) 
         
class IveoCommandManual(CommandMask):
    def  __init__(self, mask, command):
        super().__init__(IveoCommand.MANUAL, mask, command)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1]) 

class IveoCommandAutomatic(CommandMask):
    def  __init__(self, mask, command):
        super().__init__(IveoCommand.AUTOMATIC, mask, command)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class IveoCommandResult(MethodCall):
    def __init__(self, command, mask, state):
        super().__init__(IveoCommand.RESULT, [(ParameterType.INT, command), (ParameterType.BASE64, mask), (ParameterType.INT, state)])

class IveoCommandSetLabel(Command):
    def __init__(self, iveoId, label):
        super().__init__(IveoCommand.SETLABEL, [(ParameterType.INT, iveoId), (ParameterType.STRING, label)])

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class IveoCommandSetConfig(Command):
    def __init__(self, iveoId, activity, device_type):
        super().__init__(IveoCommand.SETCONFIG, [(ParameterType.INT, iveoId), (ParameterType.INT, activity), (ParameterType.INT, device_type)])
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class IveoCommandGetConfig(CommandSingle):
    def __init__(self, iveoId):
        super().__init__(IveoCommand.GETCONFIG, iveoId)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = methodResponse.parameters[0][1]
        self.activity = methodResponse.parameters[2][1]
        self.deviceType = DeviceType(int(methodResponse.parameters[3][1]))

class IveoCommandGetIds(Command):
    def __init__(self):
        super().__init__(IveoCommand.GETIDS)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)

class IveoDevice(Device):

    def __init__(self, gateway, iveoID):
        super().__init__(gateway, iveoID)
        self.communicationType = CommunicationType.IVEO
        self.deviceClass = DeviceClass.IVEO
    
    async def executeCommand(self, commandType, automatic = False):
        if automatic:
            command = IveoCommandAutomatic(self.mask, commandType)
        else:
            command = IveoCommandManual(self.mask, commandType)
        await command.execute(self.gateway)
        return command

    async def discover_properties(self):
        try:
            command = IveoCommandGetConfig(self.ID)
            await command.execute(self.gateway)
            self.device_type = command.deviceType
            self.name = command.name
            self.activity = command.activity
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))

    async def setRepeaterState(self, state):
        command = IveoCommandSetRepeater(state)
        await command.execute(self.gateway)

    async def getRepeaterState(self):
        command = IveoCommandGetRepeater()
        await command.execute(self.gateway)
        return command

    async def setIveoLabel(self):
        command = IveoCommandSetLabel(self.ID, self.name)
        await command.execute(self.gateway)

    async def setIveoConfig(self):
        command = IveoCommandSetConfig(self.ID, self.activity, self.device_type)
        await command.execute(self.gateway)

    async def getIveoConfig(self):
        command = IveoCommandGetConfig(self.ID)
        await command.execute(self.gateway)
        self.device_type = command.deviceType
        self.name = command.name
        self.activity = command.activity

    async def resetIveoChannel(self):
        command = IveoCommandFactory(self.ID)
        await command.execute(self.gateway)
        self.gateway.deleteDevice(self.ID)

    async def learnIveoChannel(self):
        command = IveoCommandLearn(self.ID)
        await command.execute(self.gateway)

    async def setToLearnMode(self):
        self.learnIveoChannel()        

    async def teachIveoChannel(self, channel):
        command = IveoCommandTeach(channel)
        _LOGGER.info("Trying to teach channel " + str(channel))
        await command.execute(self.gateway)
        if command.hasError:
            _LOGGER.info("Teaching failed for channel " + str(channel))
            return
        if await command.executed:
            _LOGGER.info("Channel " + str(channel) + "successfully taught" )

    async def manualIveoCommand(self, command):
        command = IveoCommandManual(self.mask, command)
        await command.execute(self.gateway)
        return command

    async def automaticIveoCommand(self, command):
        command = IveoCommandAutomatic(self.mask, command)
        await command.execute(self.gateway)
        return command


    async def stop(self):
        await self.automaticIveoCommand(CommandTypeIveo.STOP)

    async def moveDown(self):
        await self.automaticIveoCommand(CommandTypeIveo.DRIVEDOWN)
    
    async def moveUp(self):
        await self.automaticIveoCommand(CommandTypeIveo.DRIVEUP)
    
    async def moveIntermediatePosition1(self):
        await self.manualIveoCommand(CommandTypeIveo.POSITION_1)

    async def moveIntermediatePosition2(self):
        await self.manualIveoCommand(CommandTypeIveo.POSITION_2)
    
    async def deleteDevice(self):
        await self.resetIveoChannel()

    async def saveDevice(self):
        await self.teachIveoChannel(self.ID)


from enum import Enum

class CommeoServiceCommand(Enum):
    PING = "service.ping"
    GETSTATE = "service.getState"
    GETVERSION = "service.getVersion"
    RESET = "service.reset"
    FACTORYRESET = "service.factoryReset"
    SETLED = "service.setLED"
    GETLED = "service.getLED"
    
class CommeoParamCommand(Enum):
    SETFORWARD = "param.setForward"
    GETFORWARD = "param.getForward"
    SETEVENT = "param.setEvent"
    GETEVENT = "param.getEvent"
    GETDUTY = "param.getDuty"
    GETRF = "param.getRF"

class CommeoDeviceCommand(Enum):
    SCANSTART = "device.scanStart"
    SCANSTOP = "device.scanStop"
    SCANRESULT = "device.scanResult"
    SAVE = "device.save"
    GETIDS = "device.getIDs"
    GETINFO = "device.getInfo"
    GETVALUES = "device.getValues"
    SETFUNCTION = "device.setFunction"
    SETLABEL = "device.setLabel"
    SETTYPE = "device.setType"
    DELETE = "device.delete"
    WRITEMANUAL = "device.writeManual"
    
class CommeoSensorCommand(Enum):
    TEACHSTART = "sensor.teachStart"
    TEACHSTOP = "sensor.teachStop"
    TEACHRESULT = "sensor.teachResult"
    GETIDS = "sensor.getIDs"
    GETINFO = "sensor.getInfo"
    GETVALUES = "sensor.getValues"
    SETLABEL = "sensor.setLabel"
    DELETE = "sensor.delete"
    WRITEMANUAL = "sensor.writeManual"

class CommeoSenSimCommand(Enum):
    STORE = "senSim.store"
    DELETE = "senSim.delete"
    GETCONFIG = "senSim.getConfig"
    SETCONFIG = "senSim.setConfig"
    SETLABEL = "senSim.setLabel"
    SETVALUES = "senSim.setValues"
    GETVALUES = "senSim.getValues"
    GETIDS = "senSim.getIDs"
    FACTORY = "senSim.factory"
    DRIVE = "senSim.drive"
    SETTEST = "senSim.setTest"
    GETTEST = "senSim.getTest"
    
class CommeoSenderCommand(Enum):
    TEACHSTART = "sender.teachStart"
    TEACHSTOP = "sender.teachStop"
    TEACHRESULT = "sender.teachResult"
    GETIDS = "sender.getIDs"
    GETINFO = "sender.getInfo"
    GETVALUES = "sender.getValues"
    SETLABEL = "sender.setLabel"
    DELETE = "sender.delete"
    WRITEMANUAL = "sender.writeManual"

class CommeoGroupCommand(Enum):
    READ = "group.read"
    WRITE = "group.write"
    GETIDS = "group.getIDs"
    DELETE = "group.delete"

class CommeoCommandCommand(Enum):
    DEVICE = "command.device"
    GROUP = "command.group"
    GROUPMAN = "command.groupMan"
    RESULT = "command.result"
    
class CommeoEventCommand(Enum):
    DEVICE = "event.device"
    SENSOR = "event.sensor"
    SENDER = "event.sender"
    LOG = "event.log"
    DUTYCYCLE = "event.dutyCycle"

class IveoCommand(Enum):
    FACTORY = "iveo.factory"
    SETCONFIG = "iveo.setConfig"
    GETCONFIG = "iveo.getConfig"
    GETIDS = "iveo.getIDs"
    SETREPEATER = "iveo.setRepeater"
    GETREPEATER = "iveo.getRepeater"
    SETLABEL = "iveo.setLabel"
    TEACH = "iveo.commandTeach"
    LEARN = "iveo.commandLearn"
    MANUAL = "iveo.commandManual"
    AUTOMATIC = "iveo.commandAutomatic"
    RESULT = "iveo.commandResult"
    

class Commands(Enum):
    SERVICE = CommeoServiceCommand   
    PARAM = CommeoParamCommand
    DEVICE = CommeoDeviceCommand
    SENSOR = CommeoSensorCommand
    SENSIM = CommeoSenSimCommand
    SENDER = CommeoSenderCommand
    GROUP = CommeoGroupCommand
    COMMAND = CommeoCommandCommand
    EVENT = CommeoEventCommand
    IVEO = IveoCommand


import base64

def singlemask(id):
    
    #Obtains a base64 encoded to modify just one index
    mask =  64 * [0]
    #need to transform the position
    newid = int((id // 8) * 8  + 7 - (id % 8))    
    mask[newid] = 1
    bitstring = "".join(str(x) for x in mask)
    return base64.b64encode(bitstring_to_bytes(bitstring)).decode('utf8')

def multimask(ids):
    mask = 64 * [0]
    for id in ids:
        newid = int((id // 8) * 8  + 7 - (id % 8))    
        mask[newid] = 1
    bitstring = "".join(str(x) for x in mask)
    return base64.b64encode(bitstring_to_bytes(bitstring)).decode('utf8')


def bitstring_to_bytes(s):
    return int(s, 2).to_bytes(len(s) // 8, byteorder='big')

def b64bytes_to_bitlist(b):
    byts = base64.b64decode(b)
    return [bool(int(value)) for value in list(''.join([bin(by).lstrip('0b').zfill(8)[::-1] for by in byts]))]

def true_in_list(l):
    return [i for i,v in enumerate(l) if v]

def valueToPercentage(value):
    return int((value / 65535)*100)

def percentageToValue(perc):
    return int((65535/100)*(perc))

def intToBoolarray(value):
    return [bool(bit) for bit in '{0:10b}'.format(value)]

if __name__ == '__main__':
    for i in range (0,64):
        mask = singlemask(i)
        #print(mask)
        b = b64bytes_to_bitlist(mask)
        #print (b)
        c =true_in_list(b)
        #print (c)
        if c[0] != i:
            print("error in: " +str(i))
        else:
            print(str(i) + " OK")

    # a = 8*  bitarray('0', 'little')
    # a[7] = 1
    # print (a)
    # bitdecoded = base64.b64encode(a.tobytes())
    # print(bitdecoded)l
    # mask = singlemask(1)
    # print(mask.decode('utf-8'))


    # a = 64 * [0]
    # a[1] = 1    
    # print(a)   
    # d = "".join(str(x) for x in a)
    # print (d)
    # b = bitstring_to_bytes(d)
    # print(b)
    # c = base64.b64encode(b)  
    # print(c)

    # d = 'AQAAAAAAAAA='
    # b = base64.b64decode(d)
    # d = b'\x01\x00\x00\x00\x00\x00\x00\x00'
    # int(d)
    # test = '10000000'
    # print ("test " + base64.b64encode(test))

    # converted = [access_bit(d,i) for i in range(len(d)*8)]
    # print (converted)
    # bitarray('0',)
    # print(b)
    # c = base64.b64encode(d)
    # print(c)

    #fx4AAAAAAAA=
    #a = 'fx4AAAAAAAA='
    #print (true_in_list(b64bytes_to_bitlist(a)))




from selve.protocol import DayMode, DeviceType, DutyMode, LogType, MethodCall, MovementState, ParameterType, SensorState, lightDigital, process_response, rainDigital, senderEvents, tempDigital, windDigital
from selve.utils import *
import logging

_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)

## Commands ##

class Command(MethodCall):

    def __init__(self, method_name, parameters = []):
         super().__init__("selve.GW." + method_name.value, parameters)

class CommandSingle(Command):

    def __init__(self, method_name, iveoID):
        super().__init__(method_name, [(ParameterType.INT, iveoID)])

class CommandMask(Command):

    def __init__(self, method_name, mask, command):
        super().__init__(method_name, [(ParameterType.BASE64, mask), (ParameterType.INT, command.value)])


## Events ##

# Handling all Events

def incomingEvent(value:str):
    parameters = process_response(value)
    methodName = parameters[0][1]
    output = []
    if(methodName == "selve.GW.event.device"):
        output.name = parameters[0][1]
        output.movementState = MovementState(int(parameters[2][1]))
        output.value = valueToPercentage(int(parameters[3][1]))
        output.targetValue = valueToPercentage(int(parameters[4][1]))

        bArr = intToBoolarray(int(parameters[5][1]))
        output.unreachable = bArr[0]
        output.overload = bArr[1]
        output.obstructed = bArr[2]
        output.alarm = bArr[3]
        output.lostSensor = bArr[4]
        output.automaticMode = bArr[5]
        output.gatewayNotLearned = bArr[6]
        output.windAlarm = bArr[7]
        output.rainAlarm = bArr[8]
        output.freezingAlarm = bArr[9]
        
        output.dayMode = DayMode(int(parameters[6][1]))
        output.deviceType = DeviceType(int(parameters[7][1]))

    if(methodName == "selve.GW.event.sensor"):
        output.sensorId = int(parameters[0][1])
        output.windDigital = windDigital(int(parameters[1][1]))
        output.rainDigital = rainDigital(int(parameters[2][1]))
        output.tempDigital = tempDigital(int(parameters[3][1]))
        output.lightDigital = lightDigital(int(parameters[4][1]))
        output.sensorState = SensorState(int(parameters[5][1]))
        output.tempAnalog = int(parameters[6][1])
        output.windAnalog = int(parameters[7][1])
        output.sun1Analog = int(parameters[8][1])
        output.dayLightAnalog = int(parameters[9][1])
        output.sun2Analog = int(parameters[10][1])
        output.sun3Analog = int(parameters[11][1])

    if(methodName == "selve.GW.event.sender"):
        output.senderId = int(parameters[0][1])
        output.lastEvent = senderEvents(int(parameters[1][1]))

    if(methodName == "selve.GW.event.log"):
        output.logCode = str(parameters[0][1])
        output.logStamp = str(parameters[1][1])
        output.logValue = str(parameters[2][1])
        output.logDescription = str(parameters[3][1])
        output.logType = LogType(int(parameters[4][1]))
    
    if(methodName == "selve.GW.event.dutyCycle"):
        output.dutyMode = DutyMode(int(parameters[0][1]))
        output.rfTraffic = int(parameters[1][1])
        
    
    _LOGGER.debug("Received event " + methodName)
    output.methodName = methodName
    return output


from os import name
from selve.utils import b64bytes_to_bitlist, intToBoolarray, true_in_list, valueToPercentage
from enum import Enum, Flag
from itertools import chain
from serial.serialutil import STOPBITS_ONE
import untangle
import logging


_LOGGER = logging.getLogger(__name__)
class DeviceType(Enum):
    UNKNOWN = 0
    SHUTTER = 1
    BLIND = 2
    AWNING = 3
    SWITCH = 4
    DIMMER = 5
    NIGHT_LIGHT = 6
    DRAWN_LIGHT = 7
    HEATING = 8
    COOLING = 9
    SWITCHDAY = 10
    GATEWAY = 11

class DeviceState(Enum):
    UNUSED = 0
    USED = 1
    TEMPORARY = 2
    STALLED = 3

class MovementState(Enum):
    UNKOWN = 0
    STOPPED_OFF = 1
    UP_ON = 2
    DOWN_ON = 3

class CommunicationType(Enum):
    COMMEO = 0
    IVEO = 1
    UNKNOWN = 99

class CommandTypeIveo(Enum):
    STOP = 0
    DRIVEUP = 1
    DRIVEDOWN = 2
    POSITION_1 = 3
    POSITION_2 = 4

class CommandType(Enum):
    STOP = 0
    DRIVEUP = 1
    DRIVEDOWN = 2
    DRIVEPOS1 = 3
    SAVEPOS1 = 4
    DRIVEPOS2 = 5
    SAVEPOS2 = 6
    DRIVEPOS = 7
    STEPUP = 8
    STEPDOWN = 9 
    AUTOON = 10
    AUTOOFF = 11

class DeviceCommandTypes(Enum):
    FORCED = 0
    MANUAL = 1
    TIME = 2
    GLASS = 3

class SenSimCommands(Enum):
    STOP = 0
    UP = 1
    DOWN = 2
    POS1 = 3
    POS2 = 5

class ParameterType(Enum):
    INT = "int"
    STRING = "string"
    BASE64 = "base64"

class ScanState(Enum):
    IDLE = 0
    RUN = 1
    VERIFY = 2
    END_SUCCESS = 3
    END_FAILED = 4

class TeachState(Enum):
    IDLE = 0
    RUN = 1
    END_SUCCESS = 2

class ServiceState(Enum):
    BOOTLOADER = 0
    UPDATE = 1
    STARTUP = 2
    READY = 3

class SensorState(Enum):
    INVALID = 0
    AVAILABLE = 1
    LOW_BATTERY = 2
    COMMUNICATION_LOSS = 3
    TESTMODE = 4
    SERVICEMODE = 5

class RepeaterState(Enum):
    NONE = 0
    SINGLEREPEAT = 1
    MULTIREPEAT = 2

class LEDMode(Enum):
    OFF = 0
    ON = 1

class Forwarding(Enum):
    OFF = 0
    ON = 1

class DutyMode(Enum):
    NOT_BLOCKED = 0
    BLOCKED = 1

class DayMode(Enum):
    UNKOWN = 0
    NIGHTMODE = 1
    DAWNING = 2
    DAY = 3
    DUSK = 4

class deviceFunctions(Enum):
    SELECT = 0
    INSTALL = 1
    SENSOR = 2
    MANPROG = 3
    AUTOPROG = 4
    STOREPOSITION = 5
    DRIVEUP = 6
    DRIVEDOWN = 7
    KEYRELEASE = 8
    DRIVESTOP = 9

class LogType(Enum):
    INFO = 0
    WARNING = 1
    ERROR = 2

class DeviceClass(Enum):
    ACTOR = 0
    GROUP = 1
    SENDER = 2
    SENSIM = 3
    SENSOR = 4
    IVEO = 5
    UNKOWN = 99

## SensorVariables ##

class windDigital(Enum):
    NONE = 0
    NO_ALARM = 1
    ALARM = 2

class rainDigital(Enum):
    NONE = 0
    NO_ALARM = 1
    ALARM = 2

class tempDigital(Enum):
    NONE = 0
    NORMAL = 1
    FREEZING = 2
    HEAT = 3

class lightDigital(Enum):
    NONE = 0
    DARK = 1
    DAWN = 2
    NORMAL = 3
    LIGHT = 4

## senderEvents ##

class senderEvents(Enum):
    UNKNOWN = 0
    DRIVEUP = 1
    DRIVEDOWN = 2
    STOP = 3
    POS1 = 4
    POS2 = 5
    SAVEPOS1 = 6
    SAVEPOS2 = 7
    AUTO = 8
    MAN = 9
    NAME = 10
    KEYRELEASE = 11
    SELECT = 12
    DELETE = 13

class MethodCall:

    def __init__(self, method_name, parameters = []):
        self.method_name = method_name
        self.parameters = parameters

    def serializeToXML(self):
        xmlstr = "<methodCall>"
        xmlstr += "<methodName>"+self.method_name+"</methodName>"
        if (len(self.parameters) > 0):
            xmlstr += "<array>"
            for typ, val in self.parameters:
                xmlstr+="<{0}>{1}</{0}>".format(typ.value, val)
            xmlstr += "</array>"
        xmlstr+= "</methodCall>"
        return xmlstr.encode('utf-8')
    
    async def execute(self, gateway):
        response = await gateway.executeCommand(self)
        if response != None and isinstance(response, MethodResponse):
            self.process_response(response)
    
    def process_response(self, methodResponse):
        _LOGGER.debug(methodResponse)
        self.hasError = False
        if isinstance(methodResponse, ErrorResponse):
            self.hasError = True


class MethodResponse:

    def __init__(self, name, parameters):
        self.name = name
        self.parameters = parameters
        
class CommeoCommandResult(MethodResponse):
    def __init__(self, name, parameters):
        super().__init__(name, parameters)
        self.command = self.name
        self.commandType = DeviceCommandTypes(int(parameters[1][1]))
        self.executed = bool(parameters[2][1])
        self.successIds = [ b for b in true_in_list(b64bytes_to_bitlist(parameters[3][1]))]
        self.failedIds = [ b for b in true_in_list(b64bytes_to_bitlist(parameters[4][1]))]

class CommeoDeviceEventResponse(MethodResponse):
    def __init__(self, name, parameters):
        super().__init__(name, parameters)
        self.actorName = str(parameters[0][1])
        self.actorId = int(parameters[1][1])
        self.actorState = MovementState(int(parameters[2][1]))
        self.value = valueToPercentage(int(parameters[3][1]))
        self.targetValue = valueToPercentage(int(parameters[4][1]))
        bArr = intToBoolarray(int(parameters[5][1]))
        self.unreachable = bArr[0]
        self.overload = bArr[1]
        self.obstructed = bArr[2]
        self.alarm = bArr[3]
        self.lostSensor = bArr[4]
        self.automaticMode = bArr[5]
        self.gatewayNotLearned = bArr[6]
        self.windAlarm = bArr[7]
        self.rainAlarm = bArr[8]
        self.freezingAlarm = bArr[9]
        self.dayMode = DayMode(int(parameters[6][1]))
        self.deviceType = DeviceType(int(parameters[7][1]))

class LogEventResponse(MethodResponse):
    def __init__(self, name, parameters):
        super().__init__(name, parameters)
        self.logCode = str(parameters[0][1])
        self.logStamp = str(parameters[1][1])
        self.logValue = str(parameters[2][1])
        self.logDescription = str(parameters[3][1])
        self.logType = LogType(int(parameters[4][1]))

class DutyCycleResponse(MethodResponse):
    def __init__(self, name, parameters):
        super().__init__(name, parameters)
        self.mode = DutyMode(int(parameters[0][1]))
        self.traffic = int(parameters[1][1])

class SensorEventResponse(MethodResponse):
    def __init__(self, name, parameters):
        super().__init__(name, parameters)
        self.id = int(parameters[0][1])
        self.windDigital = windDigital(int(parameters[1][1]))
        self.rainDigital = rainDigital(int(parameters[2][1]))
        self.tempDigital = tempDigital(int(parameters[3][1]))
        self.lightDigital = lightDigital(int(parameters[4][1]))
        self.sensorState = SensorState(int(parameters[5][1]))
        self.tempAnalog = int(parameters[6][1])
        self.windAnalog = int(parameters[7][1])
        self.sun1Analog = int(parameters[8][1])
        self.dayLightAnalog = int(parameters[9][1])
        self.sun2Analog = int(parameters[10][1])
        self.sun3Analog = int(parameters[11][1])

class SenderEventResponse(MethodResponse):
    def __init__(self, name, parameters):
        super().__init__(name, parameters)
        self.senderName = str(parameters[0][1])
        self.id = int(parameters[1][1])
        self.event = senderEvents(int(parameters[2][1]))

class ErrorResponse:

    def __init__(self, message, code):
        self.message = message
        self.code = code


def create_error(obj):
    return ErrorResponse(obj.methodResponse.fault.array.string.cdata, obj.methodResponse.fault.array.int.cdata ) 

def create_response(obj):
    array = obj.methodResponse.array
    methodName = list(array.string)[0].cdata
    str_params_tmp = list(array.string)[1:]
    str_params = [(ParameterType.STRING, v.cdata) for v in str_params_tmp]
    int_params = []
    if hasattr(array, ParameterType.INT.value):
        int_params = [(ParameterType.INT, v.cdata) for v in list(array.int)]
    b64_params = []
    if hasattr(array, ParameterType.BASE64.value):
        b64_params = [(ParameterType.BASE64, v.cdata) for v in list(array.base64)]
    paramslist = [str_params, int_params, b64_params]
    flat_params_list = list(chain.from_iterable(paramslist))
    if methodName == "selve.GW.command.result":
        return CommeoCommandResult(methodName, flat_params_list)
    if methodName == "selve.GW.event.device":
        return CommeoDeviceEventResponse(name, flat_params_list)
    if methodName == "selve.GW.event.sensor":
        return SensorEventResponse(name, flat_params_list)
    if methodName == "selve.GW.event.sender":
        return SenderEventResponse(name, flat_params_list)
    if methodName == "selve.GW.event.log":
        return LogEventResponse(name, flat_params_list)
    if methodName == "selve.GW.event.dutyCycle":
        return DutyCycleResponse(name, flat_params_list)

    return MethodResponse(methodName, flat_params_list)


def process_response(xmlstr):
    _LOGGER.debug(str(xmlstr))
    #The selve device sometimes answers a badformed header. This is a patch
    xmlstr = str(xmlstr).replace('<?xml version="1.0"? encoding="UTF-8">', '<?xml version="1.0" encoding="UTF-8"?>')
    try:
        res = untangle.parse(xmlstr)
        if not hasattr(res, 'methodResponse'):
            _LOGGER.error("Bad response format")
            return None
        if hasattr(res.methodResponse, 'fault'):
            return create_error(res)
        return create_response(res)
    except Exception as e:
        _LOGGER.error("Error in XML: " + str(e) + " : " + xmlstr)


def main():

    selve_string = u'''<?xml version="1.0"? encoding="UTF-8">
    <methodResponse>
        <array>
            <string>Methodenname</string>
            <string>Parameter 1</string>
            <int>Parameter 2</int>
            <base64>Parameter 3</base64>
        </array>
    </methodResponse>
    '''

    error_string = '''<?xml version="1.0" encoding="UTF-8"?>
    <methodResponse>
        <fault>
            <array>
                <string>Method not supported!</string>                
                <int>2</int>                
            </array>
        </fault>
    </methodResponse>
    '''

    a ='<?xml version="1.0" encoding="UTF-8"?>\r\n<methodResponse>\r\n\t<array>\r\n\t\t<string>selve.GW.iveo.getIDs</string>\r\n\t\t<base64>fx4AAAAAAAA=</base64>\r\n\t</array>\r\n</methodResponse>\r\n\n'

    process_response(selve_string)


if __name__ == '__main__':
    main()



#!/usr/bin/python

import asyncio
from selve.commandClasses.command import CommeoCommandDevice, CommeoCommandGroup, CommeoCommandGroupMan
from selve.commandClasses.common import CommeoParamGetEvent, CommeoParamSetEvent, CommeoServiceFactoryReset, CommeoServiceGetState, CommeoServiceGetVersion, CommeoServicePing, CommeoServiceReset
from selve.commandClasses.sensor import CommeoSensorGetIDs, SensorDevice
from selve.commandClasses.senSim import CommeoSenSimGetIDs, SenSimDevice
from selve.commandClasses.sender import CommeoSenderGetIDs, SenderDevice
from selve.commandClasses.group import CommeoGroupGetIDs, GroupDevice
from selve.commandClasses.actor import ActorDevice, CommeoDeviceGetIDs, CommeoDeviceSave, CommeoDeviceScanResult, CommeoDeviceScanStart, CommeoDeviceScanStop
from selve.commandClasses.iveo import IveoCommandGetIds, IveoDevice
from selve.protocol import DeviceCommandTypes, ScanState, ServiceState, process_response
from selve.communication import incomingEvent

import time
import serial
import logging
import threading
import queue

_LOGGER = logging.getLogger(__name__)

class Gateway():   

    def __init__(self, port, discover = True):
        """                
        Arguments:
            port {String} -- Serial port string as it is used in pyserial
        
        Keyword Arguments:
            discover {bool} -- True if the gateway should try to discover 
                               the devices on init (default: {True})
        """
        self.port = port
        self.connected = False
        self.inputQueue = queue.Queue()
        self.outputQueue = queue.Queue()
        self._LOGGER = _LOGGER
        self.lock = threading.Lock()
        self.devices: dict = {}
        
        
        self.configserial()


        if discover:
            _LOGGER.info("Discovering devices")
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            result = loop.run_until_complete(self.discover())

        #self.readThread = threading.Thread(target=self.readFromPort)
        #self.readThread.start()

        #self.writeThread = threading.Thread(target=self.writePort)
        #self.writeThread.start()
    
    def configserial(self):
        """
        Configure the serial port
        """
        self.ser = serial.Serial(
            port=self.port,
            baudrate=115200,
            parity=serial.PARITY_NONE,
            stopbits=serial.STOPBITS_ONE,
            bytesize=serial.EIGHTBITS)
        self.ser.timeout = 0
        self.ser.xonxoff = False
        self.ser.rtscts = False
        self.ser.dsrdtr = False
        self.ser.writeTimeout = 2

    def handleData(self, data):
        incomingEvent(str(data))

    def readFromPort(self):
        while True:
            response_str = "" 
            with self.lock: 
                if not self.ser.isOpen():
                    self.ser.open()
                if self.ser.inWaiting() > 0:                        
                    response_str = "" 
                    while True:
                        response = self.ser.readline().strip()
                        response_str += response.decode()
                        if (response.decode() == ''):
                            break
                        
                    _LOGGER.debug('read data: ' + response_str)
                    self.ser.close()
                    return process_response(response_str) 
                    ## inform callback of events

                
    def writePort(self):
        while True:
            response_str = "" 
            with self.lock:
                if not self.outputQueue.empty():     
                    if not self.ser.isOpen():
                        self.ser.open()
                    try:
                        #self.ser.flushInput()
                        #self.ser.flushOutput()
                        
                        self.ser.write(self.outputQueue.get())
                        time.sleep(0.5)
                        response_str = "" 
                        while True:
                            response = self.ser.readline().strip()
                            response_str += response.decode()
                            if (response.decode() == ''):
                                break
                            
                        self.ser.close()
                        _LOGGER.info('read data: ' + response_str)
                        #return process_response(response_str)
                        # handle response somehow here
                    except Exception as e1:
                        _LOGGER.exception ("error communicating...: " + str(e1))
        

    async def executeCommand(self, command):
        """[summary]
        Execute the given command using the serial port.
        It opens a communication to the serial port each time a
        command is executed.
        At this moment it doesn't keep a queue of commands. If
        a command blocks the serial it will wait.
        
        Arguments:
            command {protocol.MethodCall} -- Command to be send 
            through the serial port
        
        Returns:
            MethodResponse -- if the command was executed 
            sucessufully
            ErrorResponse -- if the gateway returns an error
        """
        with self.lock:
            commandstr = command.serializeToXML()
            _LOGGER.info('Gateway writting: ' + str(commandstr))

            try:
                if not self.ser.isOpen():
                    self.ser.open()
                self.ser.reset_input_buffer()
                self.ser.reset_output_buffer()
                
                self.ser.write(commandstr)
                time.sleep(0.5)
                response_str = "" 
                while True:
                    response = self.ser.readline().strip()
                    response_str += response.decode()
                    if (response.decode() == ''):
                        break
                    
                _LOGGER.debug('read data: ' + response_str)            
                self.ser.close()
                return process_response(response_str)
                        
            except Exception as e:
                _LOGGER.error ("error communicating: " + str(e))

            self.ser.close()
            return None

    async def discover(self):
        """[summary]
            Discover all devices registered on the usb-commeo        
        """
        await self.gatewayReady()
            

        commandIveo = IveoCommandGetIds()
        commandCommeoActors = CommeoDeviceGetIDs()
        commandCommeoGroups = CommeoGroupGetIDs()
        commandCommeoSenders = CommeoSenderGetIDs()
        commandCommeoSenSims = CommeoSenSimGetIDs()
        commandCommeoSensors = CommeoSensorGetIDs()
        num_retries = 3
        retry_n = 0
        retry_m = 0
        while not hasattr(commandIveo, "ids") and retry_n <=num_retries:
            await commandIveo.execute(self)
            retry_n += 1
            time.sleep(1)
        retry_n = 0
        retry_m = 0
        while not hasattr(commandCommeoActors, "ids") and retry_m <=num_retries:
            await commandCommeoActors.execute(self)
            retry_m += 1
            time.sleep(1)
        retry_n = 0
        retry_m = 0
        while not hasattr(commandCommeoGroups, "ids") and retry_m <=num_retries:
            await commandCommeoGroups.execute(self)
            retry_m += 1
            time.sleep(1)
        retry_n = 0
        retry_m = 0
        while not hasattr(commandCommeoSenders, "ids") and retry_m <=num_retries:
            await commandCommeoSenders.execute(self)
            retry_m += 1
            time.sleep(1)
        retry_n = 0
        retry_m = 0
        while not hasattr(commandCommeoSenSims, "ids") and retry_m <=num_retries:
            await commandCommeoSenSims.execute(self)
            retry_m += 1
            time.sleep(1)
        retry_n = 0
        retry_m = 0
        while not hasattr(commandCommeoSensors, "ids") and retry_m <=num_retries:
            await commandCommeoSensors.execute(self)
            retry_m += 1
            time.sleep(1)


        self.devices = {}
        if not hasattr(commandIveo, "ids"):
            _LOGGER.info("Associated Iveo Devices not found") 
            iveoDevices = {}
        else:
            _LOGGER.debug(f'discover ids: {commandIveo.ids}')
            iveoDevices = dict([(id, IveoDevice(self, id) )for id in commandIveo.ids])
        
        if not hasattr(commandCommeoActors, "ids"):
            _LOGGER.info("Associated Commeo Devices not found") 
            commeoActors = {}
        else:
            _LOGGER.debug(f'discover ids: {commandCommeoActors.ids}')
            commeoActors = dict([(id, ActorDevice(self, id) )for id in commandCommeoActors.ids])
        
        if not hasattr(commandCommeoGroups, "ids"):
            _LOGGER.info("Associated Commeo Groups not found") 
            commeoGroups = {}
        else:
            _LOGGER.debug(f'discover ids: {commandCommeoGroups.ids}')
            commeoGroups = dict([(id, GroupDevice(self, id) )for id in commandCommeoGroups.ids])
        
        if not hasattr(commandCommeoSenders, "ids"):
            _LOGGER.info("Associated Commeo Devices not found") 
            commeoSenders = {}
        else:
            _LOGGER.debug(f'discover ids: {commandCommeoSenders.ids}')
            commeoSenders = dict([(id, SenderDevice(self, id) )for id in commandCommeoSenders.ids])
        
        if not hasattr(commandCommeoSenSims, "ids"):
            _LOGGER.info("Associated Commeo Devices not found") 
            commeoSenSims = {}
        else:
            _LOGGER.debug(f'discover ids: {commandCommeoSenSims.ids}')
            commeoSenSims = dict([(id, SenSimDevice(self, id) )for id in commandCommeoSenSims.ids])
        
        if not hasattr(commandCommeoSensors, "ids"):
            _LOGGER.info("Associated Commeo Devices not found") 
            commeoSensors = {}
        else:
            _LOGGER.debug(f'discover ids: {commandCommeoSensors.ids}')
            commeoSensors = dict([(id, SensorDevice(self, id) )for id in commandCommeoSensors.ids])
        

        self.devices.update(iveoDevices) 
        self.devices.update(commeoActors) 
        self.devices.update(commeoGroups) 
        self.devices.update(commeoSenders) 
        self.devices.update(commeoSenSims) 
        self.devices.update(commeoSensors)
        
        for id, device in self.devices.items():
            await device.discover_properties()
        
        self.list_devices() 
       
    def addDevice(self, id, device):
        self.devices = self.devices.update({id:device})

    def deleteDevice(self, id):
        if self.is_id_registered(id):
            del self.devices[id]

    def is_id_registered(self, id):
        """[summary]
        check if a device id is registered on the gateway
        Arguments:
            id {int} -- Device id to check
        
        Returns:
            boolean -- True if the id is registered
                       False otherwise
        """
        return id in self.devices
        
    def findFreeId(self):
        i = 0
        while i < 64:
            if not self.is_id_registered(i):
                return i
            i=i+1

    def list_devices(self):
        """[summary]
        Print the list of registered devices
        """ 
        for id, device in self.devices.items():
            #print(str(device))
            _LOGGER.info(str(device))
           
    ## Common ##
    async def pingGateway(self):
        self.gatewayReady()
        command = CommeoServicePing()
        await command.execute(self)
        print("Ping")

    # GATEWAY STATE....

    async def gatewayState(self):
        command = CommeoServiceGetState()
        await command.execute(self)
        if hasattr(command, "status"):
            return command.status

    async def gatewayReady(self):
        state = await self.gatewayState() 
        if state == "READY":
            return
        else:
            _LOGGER.exception("Gateway state error: " + state)
            raise GatewayError

    async def getVersionG(self):
        self.gatewayReady()
        command = CommeoServiceGetVersion()
        return await command.execute(self)

    def getGatewayFirmwareVersion(self):
        command = self.getVersionG()
        if hasattr(command, "version"):
            return command.version

    def getGatewaySerial(self):
        command = self.getVersionG()
        if hasattr(command, "serial"):
            return command.serial

    def getGatewaySpec(self):
        command = self.getVersionG()
        if hasattr(command, "spec"):
            return command.spec

    async def resetGateway(self):
        self.gatewayReady()
        command = CommeoServiceReset()
        await command.execute(self)
        while not command.executed:
            time.sleep(1)
        command = CommeoServiceGetState()
        await command.execute(self)
        retries = 0
        while command.status != ServiceState.READY:
            retries = retries+1
            await command.execute(self)
            if retries == 5:
                break
            time.sleep(3)
        if command.status != ServiceState.READY:
            _LOGGER.info("Error: Gateway could not be reset or loads too long")
        else:
            _LOGGER.info("Gateway reset")

    async def factoryResetGateway(self):
        self.gatewayReady()
        command = CommeoServiceFactoryReset()
        await command.execute(self)
        if command.executed:
            _LOGGER.info("Factory reset successful")
        else:
            _LOGGER.info("Factory reset failed")

    async def setEvents(self, eventDevice, eventSensor, eventSender, eventLogging, eventDuty):
        self.gatewayReady()
        command = CommeoParamSetEvent(eventDevice, eventSensor, eventSender, eventLogging, eventDuty)
        command.execute(self)

    async def getEvents(self):
        self.gatewayReady()
        command = CommeoParamGetEvent()
        await command.execute(self)
        _LOGGER.debug("Events: " + str(command.eventDevice) + " " + str(command.eventSensor) + " " + str(command.eventSender) + " " + str(command.eventLogging) + " " + str(command.eventDuty))

## Actor

    async def sendCommandToActor(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
        self.gatewayReady()
        command = CommeoCommandDevice(id, command, type, parameter)
        await command.execute(self)
        return command.executed

    async def scanActorDevices(self):
        self.gatewayReady()
        commandStart = CommeoDeviceScanStart()
        commandStop = CommeoDeviceScanStop()
        commandResult = CommeoDeviceScanResult()

        await commandStart.execute(self)
        await commandResult.execute(self)
        while commandResult.scanState == ScanState.RUN or commandResult.scanState == ScanState.VERIFY:
            await commandResult.execute(self)
            time.sleep(1)
        if commandResult.scanState == ScanState.END_SUCCESS:
            if commandResult.noNewDevices > 0:
                return commandResult.foundIds

        return {}

    async def saveActorDevices(self, ids):
        self.gatewayReady()
        if len(ids) > 0:
            for id in ids:
                commandSave = CommeoDeviceSave(id)
                await commandSave.execute(self)
                dev = ActorDevice(self, id , True)
                self.addDevice(id, dev)
                dev.saveDevice()

    async def deleteActorDevice(self, id):
        if self.is_id_registered(id):
            dev:ActorDevice = self.devices[id]
            dev.deleteDevice()

## Group

    async def sendCommandToGroup(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
        self.gatewayReady()
        command = CommeoCommandGroup(id, command, type, parameter)
        await command.execute(self)
        return command.executed

    async def sendCommandToGroupMan(self, idMask, command, type=DeviceCommandTypes.MANUAL, parameter=0):
        self.gatewayReady()
        command = CommeoCommandGroupMan(command, type, idMask, parameter)
        await command.execute(self)
        return command.ids


class GatewayError(Exception):
    pass



from selve.utils import b64bytes_to_bitlist, true_in_list
import logging
from selve.protocol import DeviceCommandTypes, ParameterType
from selve.commands import CommeoCommandCommand
from selve.communication import Command
_LOGGER = logging.getLogger(__name__)


class CommeoCommandDevice(Command):
    def __init__(self, deviceId, command, commmandType = DeviceCommandTypes.MANUAL, parameter = 0):
        super().__init__(CommeoCommandCommand.DEVICE, [(ParameterType.INT, deviceId), (ParameterType.INT, command.value), (ParameterType.INT, commmandType.value), (ParameterType.INT, parameter)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoCommandGroup(Command):
    def __init__(self, deviceId, command, commmandType = DeviceCommandTypes.MANUAL, parameter = 0):
        super().__init__(CommeoCommandCommand.GROUP, [(ParameterType.INT, deviceId), (ParameterType.INT, command.value), (ParameterType.INT, commmandType.value), (ParameterType.INT, parameter)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoCommandGroupMan(Command):
    def __init__(self, command, commandType, deviceIdMask, parameter = 0):
        super().__init__(CommeoCommandCommand.GROUPMAN, [(ParameterType.INT, command.value), (ParameterType.INT, commandType.value), (ParameterType.BASE64, deviceIdMask), (ParameterType.INT, parameter)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)





# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    
    name='python-selve-new',  # Required    
    version='1.0.23',  # Required  
    description='Python library for interfacing with selve devices using the USB-RF controller; forked from GPULIDO',  # Required   
    long_description=long_description,  # Optional    
    url='https://github.com/Kannix2005/python-selve',  # Optional
    author='Stefan Altheimer',  # Optional
   
    author_email='stefan.altheimer@gmail.com',  # Optional

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[  # Optional        
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
   
    keywords='selve blind awning shutter usb rf',  # Optional

    packages=find_packages(),  # Required

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'pyserial',
        'pybase64',
        'untangle'
        ],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    entry_points={  # Optional
        'console_scripts': [
           # 'sample=sample:main',
        ],
    },
)


from selve.commandClasses.sender import CommeoSenderGetInfo
from selve.device import Device
from selve.communication import Command, CommandSingle
from selve.protocol import CommunicationType, DeviceClass, lightDigital, rainDigital, tempDigital, windDigital
from selve.protocol import ParameterType
from selve.commands import CommeoSenSimCommand
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
import logging


_LOGGER = logging.getLogger(__name__)


class CommeoSenSimStore(Command):
    def __init__(self, deviceId, senSimId):
        super().__init__(CommeoSenSimCommand.STORE, [(ParameterType.INT, senSimId), (ParameterType.INT, deviceId)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
class CommeoSenSimDelete(Command):
    def __init__(self, deviceId, senSimId):
        super().__init__(CommeoSenSimCommand.DELETE, [(ParameterType.INT, senSimId), (ParameterType.INT, deviceId)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
class CommeoSenSimGetConfig(Command):
    def __init__(self, senSimId):
        super().__init__(CommeoSenSimCommand.GETCONFIG, [(ParameterType.INT, senSimId)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = str(methodResponse.parameters[0][1])
        self.senSimId = int(methodResponse.parameters[1][1])
        self.activity = bool(methodResponse.parameters[2][1])

class CommeoSenSimSetConfig(Command):
    def __init__(self, senSimId, activate):
        super().__init__(CommeoSenSimCommand.SETCONFIG, [(ParameterType.INT, senSimId), (ParameterType.INT, activate)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenSimSetLabel(Command):
    def __init__(self, senSimId, name):
        super().__init__(CommeoSenSimCommand.SETLABEL, [(ParameterType.INT, senSimId), (ParameterType.INT, name)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenSimSetValues(Command):
    def __init__(self, senSimId, windDigital, rainDigital, tempDigital, lightDigital, tempAnalog, windAnalog, sun1Analog, dayLightAnalog, sun2Analog, sun3Analog):
        super().__init__(CommeoSenSimCommand.SETVALUES, [(ParameterType.INT, senSimId), (ParameterType.INT, windDigital), (ParameterType.INT, rainDigital), (ParameterType.INT, tempDigital), (ParameterType.INT, lightDigital), (ParameterType.INT, tempAnalog), (ParameterType.INT, windAnalog), (ParameterType.INT, sun1Analog), (ParameterType.INT, dayLightAnalog), (ParameterType.INT, sun2Analog), (ParameterType.INT, sun3Analog)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenSimGetValues(CommandSingle):
    def __init__(self, senSimId):
        super().__init__(CommeoSenSimCommand.GETVALUES, senSimId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.windDigital = windDigital(int(methodResponse.parameters[1][1]))
        self.rainDigital = rainDigital(int(methodResponse.parameters[2][1]))
        self.tempDigital = tempDigital(int(methodResponse.parameters[3][1]))
        self.lightDigital = lightDigital(int(methodResponse.parameters[4][1]))
        self.tempAnalog = int(methodResponse.parameters[5][1])
        self.windAnalog = int(methodResponse.parameters[6][1])
        self.sun1Analog = int(methodResponse.parameters[7][1])
        self.dayLightAnalog = int(methodResponse.parameters[8][1])
        self.sun2Analog = int(methodResponse.parameters[9][1])
        self.sun3Analog = int(methodResponse.parameters[10][1])

class CommeoSenSimGetIDs(Command):
    def __init__(self):
        super().__init__(CommeoSenSimCommand.GETIDS)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)

class CommeoSenSimFactory(CommandSingle):
    def __init__(self, senSimId):
        super().__init__(CommeoSenSimCommand.FACTORY, senSimId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenSimDrive(Command):
    def __init__(self, senSimId, command):
        super().__init__(CommeoSenSimCommand.DRIVE, [(ParameterType.INT, senSimId), (ParameterType.INT, command)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenSimSetTest(Command):
    def __init__(self, senSimId, testMode):
        super().__init__(CommeoSenSimCommand.SETTEST, [(ParameterType.INT, senSimId), (ParameterType.INT, testMode)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenSimGetTest(CommandSingle):
    def __init__(self, senSimId):
        super().__init__(CommeoSenSimCommand.GETTEST)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.testMode = bool(methodResponse.parameters[1][1])
    
class SenSimDevice(Device):
    def __init__(self, gateway, id):
        super().__init__(gateway, id)
        self.communicationType = CommunicationType.COMMEO
        self.deviceClass = DeviceClass.SENSIM

    def discover_properties(self):
        try:
            command = CommeoSenderGetInfo(self.ID)
            command.execute(self.gateway)
            self.device_type = command.deviceType
            self.name = command.name
            self.rfAddress = command.rfAddress
            self.state = command.state
            self.deviceClass = DeviceClass.SENSIM
            self.communicationType = CommunicationType.COMMEO
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))


    def storeSenSim(self, deviceid, sensimid):
        command = CommeoSenSimStore(deviceid, sensimid)
        command.execute(self.gateway)

    def deleteSenSim(self, deviceId, senSimId):
        command = CommeoSenSimDelete(deviceId, senSimId)
        command.execute(self.gateway)

    def getSenSimConfig(self, id):
        command = CommeoSenSimGetConfig(id)
        command.execute(self.gateway)

    def setSenSimConfig(self, id, activity=True):
        command = CommeoSenSimSetConfig(id, activity)
        command.execute(self.gateway)

    def setSenSimLabel(self, id, name):
        command = CommeoSenSimSetLabel(id, name)
        command.execute(self.gateway)

    def setSenSimValues(self, id, values):
        command = CommeoSenSimSetValues(id, values.windDigital, values.rainDigital, values.tempDigital, values.lightDigital, values.tempAnalog, values.windAnalog, values.sun1Analog, values.dayLightAnalog, values.sun2Analog, values.sun2Analog)
        command.execute(self.gateway)

    def getSenSimValues(self, id):
        command = CommeoSenSimGetValues(id)
        command.execute(self.gateway)
        return command

    def resetSenSimToFactory(self, id):
        command = CommeoSenSimFactory(id)
        command.execute(self.gateway)

    def driveSenSim(self, id, commandV):
        command = CommeoSenSimDrive(id, commandV)
        command.execute(self.gateway)

    def setTestSenSimOn(self, id):
        command = CommeoSenSimSetTest(id, 1)
        command.execute(self.gateway)

    def setTestSenSimOff(self, id):
        command = CommeoSenSimSetTest(id, 0)
        command.execute(self.gateway)

    def getTestSenSim(self, id):
        command = CommeoSenSimGetTest(id)
        command.execute(self.gateway)
        return command.testMode



from selve.commandClasses.command import CommeoCommandGroup
from selve.device import Device
from selve.protocol import CommandType, CommunicationType, DeviceClass, DeviceCommandTypes, ParameterType
from selve.utils import b64bytes_to_bitlist, true_in_list
import logging
from selve.commands import CommeoDeviceCommand, CommeoGroupCommand
from selve.communication import Command, CommandSingle
_LOGGER = logging.getLogger(__name__)


class CommeoGroupRead(CommandSingle):
    def __init__(self, groupId):
        super().__init__(CommeoGroupCommand.READ, groupId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[1][1]))]
        _LOGGER.debug(self.ids)
        self.name = str(methodResponse.parameters[2][1])

class CommeoGroupWrite(Command):
    def __init__(self, groupId, actorIdMask, name):
        super().__init__(CommeoGroupCommand.WRITE, [(ParameterType.INT, groupId), (ParameterType.BASE64, actorIdMask), (ParameterType.STRING, name)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoGroupGetIDs(Command):
    def __init__(self):
        super().__init__(CommeoGroupCommand.GETIDS)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)

class CommeoGroupDelete(CommandSingle):
    def __init__(self, groupId):
        super().__init__(CommeoGroupCommand.DELETE, groupId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class GroupDevice(Device):
    def __init__(self, gateway, id):
        super().__init__(gateway, id)
        self.communicationType = CommunicationType.COMMEO
        self.deviceClass = DeviceClass.GROUP

    def discover_properties(self):
        try:
            command = CommeoGroupRead(self.ID)
            command.execute(self.gateway)
            self.device_type = "GROUP"
            self.name = command.name
            self.deviceClass = DeviceClass.GROUP
            self.communicationType = CommunicationType.COMMEO
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))


    def readGroup(self, id):
        command = CommeoGroupRead(id)
        command.execute(self.gateway)
        return command

    def writeGroup(self, id, idMask, name):
        command = CommeoGroupWrite(id, idMask, name)
        command.execute(self.gateway)

    def deleteGroup(self, id):
        command = CommeoGroupDelete(id)
        command.execute(self.gateway)

    def executeCommand(self, command, commandType = DeviceCommandTypes.MANUAL, parameter=0):
        command = CommeoCommandGroup(self.ID, command, commandType, parameter)
        command.execute(self.gateway)
        return command
    
    def stop(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.STOP, type)

    def moveDown(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.DRIVEDOWN, type)
    
    def moveUp(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.DRIVEUP, type)
    
    def moveIntermediatePosition1(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.DRIVEPOS1, type)

    def moveIntermediatePosition2(self, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.DRIVEPOS2, type)
    
    def driveToPos(self, position, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.DRIVEPOS, type, position)

    def stepUp(self, degrees, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.STEPUP, type, degrees)

    def stepDown(self, degrees, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        self.executeCommand(CommandType.STEPDOWN, type, degrees)

    def setAutomatic(self, autoOn, forced=False):
        if forced:
            type=DeviceCommandTypes.FORCED
        else:
            type=DeviceCommandTypes.MANUAL

        if autoOn:
            self.executeCommand(CommandType.AUTOON, type)
        else:
            self.executeCommand(CommandType.AUTOOFF, type)
